# dispframe - calculates post-pre integration image and displays the result
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 06. Sep. 2001
##############################################################################
procedure idispframe(image,frame)

file	image	{prompt="Image to be displayed"}
int	frame	{prompt="Frame to be written into"}

bool	flatfield {prompt="Flatfield image before display ?"}
file	flat	{prompt="Name of Flatfield image"}

bool	subsky	{prompt="subtract sky image before display ?"}
file	sky	{prompt="Name of Sky image"}

bool	stat	{prompt="show image statistics ?"}

# the following parameters are from/for the display task
file	bpmask
string	bpdisplay
string	bpcolors
file	overlay
string	ocolors
bool	erase
bool	border_erase
bool	select_frame
bool	repeat
bool	fill
bool	zscale
real	contrast
bool	zrange
file	zmask
int	nsample
real	xcenter
real	ycenter
real	xsize
real	ysize
real	xmag
real	ymag
int	order
real	z1
real	z2
string	ztrans
file	lutfile

string  *list1  {prompt="Ignore this parameter(list1)"}

begin

  file		diffile,skydiff,flatdiff
  string	imgname,stmp,junk
  int		imgframe,nsc
  real		exptim,emode,mode1,mode2,emean,mean1,mean2
  bool		delf,dels,bsub

  # need to copy arguments to define them for sure
  imgname=image
  imgframe=frame

  # set default values for variables
  flatdiff = ""
  skydiff = ""
  delf = no
  dels = no
  bsub = no

  # clean up frames that (might have) been left around from last run
  imdelete(images="tmp$dispframe*",
           go_ahead = yes,
           verify=no,
           >>& "dev$null")

  # make sure flatfield image is post-pre subtracted
  if ( flatfield )
    {
    # check file subtraction and format
    icheckfile(flat,verbose=no)

    # check if sky frame is post-pre subtracted. If not, do it.
    if ( icheckfile.subtract )
      {
      flatdiff = flat
      }
    else
      {
      flatdiff = mktemp ("tmp$dispframe")
      delf = yes

      # calculate post-pre image
      print ("calculating post-pre image of flatfield ...") 
      iframediff (input=flat,
 	          output=flatdiff,
 	          subdark=no,
 	          flatfield=no,
 	          subsky=no,
 	          wcs=no,
 	          info=no)
      }
    }

  # make sure sky image is post-pre subtracted
  if ( subsky )
    {
    # check file subtraction and format
    icheckfile(sky,verbose=no)

    # check if sky frame is post-pre subtracted. If not, do it.
    if ( icheckfile.subtract )
      {
      skydiff = sky
      }
    else
      {
      skydiff = mktemp ("tmp$dispframe")
      delf = no

      # calculate post-pre image
      print ("calculating post-pre image of sky image ...") 
      iframediff (input=sky,
 	          output=skydiff,
 	          subdark=no,
 	          flatfield=no,
 	          subsky=no,
 	          wcs=no,
 	          info=no)
      }
    }

  # check file subtraction and format and store result
  icheckfile(imgname,verbose=no)
  bsub = icheckfile.subtract

  # check if image is post-pre subtracted. If not, do it.
  if ( bsub && ! subsky )
    {
    diffile=imgname
    }
  else
    {
    diffile = mktemp ("tmp$dispframe")

    # calculate post-pre image
    iframediff (input=imgname,
 	        output=diffile,
 	        subdark=no,
 	        flatfield=flatfield,
 	        flatimg=flatdiff,
 	        subsky=subsky,
 	        skyimg=skydiff,
 	        wcs=yes,
 	        info=yes)
    }

  # call display to display the image
  display (image=diffile, \
	   frame=imgframe, \
	   bpmask=bpmask, \
	   bpdisplay=bpdisplay, \
	   bpcolors=bpcolors, \
	   overlay=overlay, \
	   ocolors=ocolors, \
	   erase=erase, \
	   border_erase=border_erase, \
	   select_frame=select_frame, \
	   repeat=repeat, \
	   fill=fill, \
	   zscale=zscale, \
	   contrast=contrast, \
	   zrange=zrange, \
	   zmask=zmask, \
	   nsample=nsample, \
	   xcenter=xcenter, \
	   ycenter=ycenter, \
	   xsize=xsize, \
	   ysize=ysize, \
	   xmag=xmag, \
	   ymag=ymag, \
	   order=order, \
	   z1=z1, \
	   z2=z2, \
	   ztrans=ztrans, \
	   lutfile=lutfile)

  # print statistical information if asked for
  if ( stat )
    {
    if ( bsub )
      {
      istatistics(input=imgname,
                  sections="data",
                  fields="image,mode,mean,stddev,min,max",
                  lower=INDEF,
                  upper=INDEF,
                  binwidth=0.1,
                  format=yes)
      }
    else
      {
      stmp = mktemp ("tmp$dispframe")

      istatistics(\
        input=imgname,
        sections="pre,post",
        fields="image,mode,mean,stddev,min,max",
        lower=INDEF,
        upper=INDEF,
        binwidth=0.1,
        format=yes, >stmp)

      # show result
      type(stmp)

      # work out exposure time limit = sky saturation
      list1 = stmp
      nsc = fscan(list1, junk)
      nsc = fscan(list1, junk, mode1, mean1)
      nsc = fscan(list1, junk, mode2, mean2)

      # get exposure time
      ifileinfo(imgname,verbose=no)
      exptim = ifileinfo.exptim

      print("\nSky Saturation Exposure Time Limit [s]:")
      if ( mode2 > mode1 )
        {
        emode = (30000.-mode1)*exptim/(mode2-mode1)
        printf("  from mode : %8.2f\n",emode)
        }
      else
        {
        print("  from mode : N/A")
        }
      if ( mean2 != mean1 )
        {
        emean = (30000.-mean1)*exptim/(mean2-mean1)
        printf("  from mean : %8.2f\n",emean)
        }
      else
        {
        print("  from mean : N/A")
        }

      delete(stmp,yes,verify=no)
      }
    }
  
  # don't clean up temporary file of displayed image now so that imexam can
  # still work. Cleanup is done next time idispframe is executed. This is not
  # nice, but is a quick fix to provide what people expect, namely to display
  # a frame and then run imexam on it.
  # Other temporary files can of course be removed.
  if ( delf ) imdelete(images=flatdiff, go_ahead=yes, verify=no, >>& "dev$null")
  if ( dels ) imdelete(images=skydiff, go_ahead=yes, verify=no, >>& "dev$null")

  # clear list variables
  list1 = ""

end
