# idomeflat - generate flat field from dome flatfield images
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 10. Apr. 2000
##############################################################################
procedure idomeflat(flats,background,output)

string	flats		{prompt="List of flatfield exposures (lamp on)"}
string	background	{prompt="List of flatfield exposures (lamp off)"}
string	output		{prompt="Name of output file"}
bool	verbose		{prompt="Verbose ?"}

begin
  file		fon,foff,ftmp
  string	inf,inb,ino
  string	log

  # copy input arguments to force definition at the beginning of the script
  inf = flats
  inb = background
  ino = output

  # decide if to show output
  if ( verbose )
    log = "STDOUT"
  else
    log = ""

  # create temporary file to hold averaged flats (light on)
  fon = mktemp ( "tmp$fon" )

  # combine flats (light on)
  imcombine (\
        input = inf,
       output = fon,
      rejmask = "",
       plfile = "",
        sigma = "",
      logfile = log,
      combine = "average",
       reject = "sigclip",
      project = no,
      outtype = "real",
      offsets = "none",
     masktype = "none",
    maskvalue = 0.,
        blank = 1.,
        scale = "exposure",
         zero = "none",
       weight = "none",
      statsec = "",
      expname = "EXPTIM",
   lthreshold = INDEF,
   hthreshold = INDEF,
         nlow = 1,
        nhigh = 1,
        nkeep = 1,
        mclip = yes,
       lsigma = 3.,
       hsigma = 3.,
      rdnoise = "0.",
         gain = "1.",
       snoise = "0.",
     sigscale = 0.1,
        pclip = -0.5,
         grow = 0.)

  # create temporary file to hold averaged flats (light off)
  foff = mktemp ( "tmp$foff" )

  # combine flats (light off)
  imcombine (\
        input = inb,
       output = foff,
      rejmask = "",
       plfile = "",
        sigma = "",
      logfile = log,
      combine = "average",
       reject = "sigclip",
      project = no,
      outtype = "real",
      offsets = "none",
     masktype = "none",
    maskvalue = 0.,
        blank = 1.,
        scale = "exposure",
         zero = "none",
       weight = "none",
      statsec = "",
      expname = "EXPTIM",
   lthreshold = INDEF,
   hthreshold = INDEF,
         nlow = 1,
        nhigh = 1,
        nkeep = 1,
        mclip = yes,
       lsigma = 3.,
       hsigma = 3.,
      rdnoise = "0.",
         gain = "1.",
       snoise = "0.",
     sigscale = 0.1,
        pclip = -0.5,
         grow = 0.)

  # create temporary file to hold averaged flats (light on)
  ftmp = mktemp ( "tmp$dometmp" )

  # subtract dark current image from dome flat image
  imarith (\
     operand1 = fon,
           op = "-",
     operand2 = foff,
       result = ftmp,
        title = "INGRID Dome Flat",
      divzero = 0.,
      hparams = "",
      pixtype = "",
     calctype = "",
      verbose = verbose,
        noact = no)

  # check file subtraction and format
  icheckfile(ftmp,verbose=no)

  # check if result needs to be post-pre subtracted
  if ( ! icheckfile.subtract )
    {
    # calculate post-pre image
    if ( verbose ) print ("calculating post-pre image ...") 
    iframediff (input=ftmp, output=ftmp)
    }

  # devide image by median
  if ( verbose ) print ("normalizing image ...") 
  imexpr("a/median(a)",ino,ftmp,
         dims="auto",intype="auto",outtype="auto",
         refim="auto",verbose=verbose)

  imdelete(fon,yes,verify=no)
  imdelete(foff,yes,verify=no)
  imdelete(ftmp,yes,verify=no)

end
