# ifinddark - find dark current
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 08. Jul. 2001
##############################################################################
procedure ifinddark(input)

string	input			{prompt="List of dark images"}

string	outtab			{prompt="Name of output table"}

string	section = "[*,*]"	{prompt="Selected image section"}
int	xdiv			{prompt="number of divisions in x"}
int	ydiv			{prompt="number of divisions in y"}

real    clip = 3		{prompt="clipping factor in sigma"}
string	center = "midpt"	{prompt="Central statistical measure",
				    enum="mean|midpt|mode"}
string	weighting="instrumental" {prompt="Weighting for fit",
				    enum="uniform|instrumental|statistical"}

bool	plot = yes		{prompt="plot the result ?"}
bool	verbose			{prompt="verbose"}

string *list1			{prompt="Ignore this parameter(list1)"}
string *list2			{prompt="Ignore this parameter(list2)"}
string *list3			{prompt="Ignore this parameter(list3)"}

begin
  string	ini
  string	fname,instr,sfname,stmp1,stmp2,trim,stats1,stats2,fit
  string	tmptab,table,name,filsec
  file		tmpi,tmpl,timf,btmp
  real		etime,m,s,rtmp1,rtmp2,low,high,dark,darkerr
  int		ip,xs,xe,xdp,ys,ye,ydp
  bool		first

  # copy input arguments to force definition at the beginning of the script
  ini = input

  # setup of variables
  first = yes

  # generate unique (base) temporary file name
  btmp = mktemp("tmp$ifinddark")

  # generate unique temporary file names
  tmpl = btmp//"-tmpl"
  fit = btmp//"-fit"

  # check if output table already exists
  if ( outtab != "" && access(outtab) )
    {
    # table exists. Inform user
    print("")
    print("INFO: output table "//outtab//" found.")
    print("INFO: Results are taken from the existing output table.")
    print("INFO: Remove the output table to re-calculate the statistics.")
    print("")

    # create input file for fitting
    tdump(table = outtab,
          cdfile = "",
          pfile = "",
          datafile = "STDOUT",
          columns = "exptim,mean,stdev",
          rows = "-",
          pwidth = 80, >tmpl)
    }
  else
    {
    # output table does not exist. We have to calculate all the results

    # create unique temporary file names
    tmpi = btmp//"-tmpi"
    timf = btmp//"-timf"

    # create temporary file to hold list of image sections to find
    # statistics for
    filsec = btmp//"-filsec"
    stats1 = btmp//"-stats1"
    stats2 = btmp//"-stats2"

    # create temporary table names
    table = btmp//"-table"
    tmptab = btmp//"-tmptab.tab"

    # expand image name.
    sections(ini,option="fullname",> tmpl)
  
    # Work through list of input files
    list1 = tmpl
    while (fscan(list1, fname) != EOF)
      {
      # find exposure time
      ifileinfo(input=fname, verbose=no)
      etime = ifileinfo.exptim
  
      # check file subtraction and format
      icheckfile(fname,verbose=no)

      # check if data file needs to be post-pre subtracted
      if ( icheckfile.subtract )
        {
        print ("working on image "//fname//" ...")
        sfname = fname
        }
      else
        {
        # delete previous temporary image
        imdelete(images=tmpi, go_ahead=yes, verify=no, >>& "/dev/null")

        # calculate post-pre image
        print ("calculating post-pre for image "//fname//" ...")
        iframediff (input = fname,
                    output = tmpi,
                    subdark = no,
                    flatfield = no,
                    subsky = no,
                    wcs = no,
                    verbose = no,
                    info = no )
        sfname = tmpi
        }

      if ( first )
        {
        # get size of useful area from image header if the section was not
        # specified directly
        ip = stridx("[",section)
        if ( ip == 0 )
          {
          imgets (image=sfname, param=section)
          trim = imgets.value
          }
        else
          {
          trim = section
          }

        # extract section bounding values
        # first, split the x and y sections
        # then expand * if necessary
        # last, assign the section start and end values
        ip = stridx(",", trim)
        stmp1 = substr(trim, 2, ip-1)
        stmp2 = substr(trim, ip+1, strlen(trim)-1)

        ip = stridx(":", stmp1)
        if ( ip == 0 )
          {
          xs = 1
          imgets (image=sfname, param="i_naxis1")
          xe = int(imgets.value)
          }
        else
          {
          xs = int(substr(stmp1,1,ip-1))
          xe = int(substr(stmp1,ip+1,strlen(stmp1)))
          }
        xdp = int( (xe-xs+1) / xdiv )
        if ( verbose ) {print ( "x step size : "//xdp//" pixels" )}

        ip = stridx(":", stmp2)
        if ( ip == 0 )
          {
          ys = 1
          imgets (image=sfname, param="i_naxis2")
          ye = int(imgets.value)
          }
        else
          {
          ys = int(substr(stmp2,1,ip-1))
          ye = int(substr(stmp2,ip+1,strlen(stmp2)))
          }
        ydp = int( (ye-ys+1) / ydiv )
        if ( verbose ) {print ( "y step size : "//ydp//" pixels" )}

        first = no
        }

      # loop over x divisions
      for (i=1; i<=xdiv; i+=1)
        {
        stmp2 = sfname//"["//(xs+(i-1)*xdp)//":"//(xs+i*xdp-1)//","

        # loop over y divisions
        for (j=1; j<=ydiv; j+=1)
          {
          print (stmp2//(ys+(j-1)*ydp)//":"//(ys+j*ydp-1)//"]", >> filsec)
          }
        }

      # calculate image statistics for all sections
      if ( verbose ) {print ("calculating statistics ...")}
      imstatistics ("@"//filsec,
                    fields="midpt,stddev",
                    lower=INDEF,
                    upper=INDEF,
                    binwidth=0.1,
                    format=no, > stats1)

      # re-calculate statistics with sigma clipping
      if (verbose )
        {print ("recalculating statistics with "//clip//" sigma clipping ...")}
      list2 = filsec
      list3 = stats1
      while ( fscan(list2, name) == 1 )
        {
        if (fscan (list3, rtmp1, rtmp2) != 2) goto err
        low = rtmp1 - clip*rtmp2
        high = rtmp1 + clip*rtmp2
        imstatistics (name, \
                      fields ="midpt,stddev", \
                      lower = low, \
                      upper = high, \
                      binwidth = 0.1, \
                      format = no, >> stats2)
        }

ererr:
      list2 = ""
      list3 = ""

      # read statistics into table
      tcreate (table = tmptab, \
               cdfile = "ingrid_ql$ifinddark1.cd", \
               datafile = stats2, \
               uparfile = "", \
               nskip = 0, \
               nlines = 0, \
               nrows = 0, \
               hist = no, \
               extrapar = 0, \
               tbltype = "default", \
               extracol = 0 )

      # repeate stringent sigma clipping on final table n times
      low = INDEF
      high = INDEF
      for (i=1; i<=3; i+=1)
        {
        # calculate average mean and stdev
        tstat (intable = tmptab, \
               column = "mean", \
               outtable = "", \
               lowlim = low,\
               highlim = high )

        # find limits for recalculation of final statistics
        if ( center == "mean" )
          {
          low = tstat.mean - 1*tstat.stddev
          high = tstat.mean + 1*tstat.stddev
          }
        else
          {
          low = tstat.median - 1*tstat.stddev
          high = tstat.median + 1*tstat.stddev
          }
        }

      # last recalculation with more relaxed sigma clipping again.
      if ( center == "mean" )
        {
        low = tstat.mean - 2*tstat.stddev
        high = tstat.mean + 2*tstat.stddev
        }
      else
        {
        low = tstat.median - 2*tstat.stddev
        high = tstat.median + 2*tstat.stddev
        }

      tstat (intable = tmptab, \
             column = "mean", \
             outtable = "", \
             lowlim = low,\
             highlim = high )
      m = tstat.mean
      s = tstat.stddev

      # plot results if requested
      if ( plot )
        {
        axispar.xlabel = "Median ADU"
        axispar.ylabel = "Number"
        axispar.title = "Histogram for Median of "//fname//section//" "//str(m)
        thistogram (tmptab,"STDOUT","mean") | sgraph (crvstyle="pseudohist")
        }

# #    tstat (intable = tmptab, \
# #           column = "stdev", \
# #           outtable = "", \
# #           lowlim = INDEF,\
# #           highlim = INDEF )
# #    s = tstat.mean

      # write exposure time to temporary file
      print(etime//" "//m//" "//s, >> timf)

      # print verbose information
      if ( verbose ) print(fname//" : "//etime//" "//m//" "//s)

      # now delete temporary files
      delete(tmptab,yes,verify=no)
      delete(stats1,yes,verify=no)
      delete(stats2,yes,verify=no)
      delete(filsec,yes,verify=no)
      }

    # sort exposure times
    delete(tmpl,yes,verify=no)
    sort(input_file = timf,
         column = 0,
         ignore_white = no,
         numeric_sort = yes,
         reverse_sort = no,
         > tmpl)

    # write results to output table if specified
    if ( outtab != "" )
      {
      # read statistics into table
      tcreate (table = outtab, \
               cdfile = "ingrid_ql$ifinddark2.cd", \
               datafile = tmpl, \
               uparfile = "", \
               nskip = 0, \
               nlines = 0, \
               nrows = 0, \
               hist = no, \
               extrapar = 0, \
               tbltype = "default", \
               extracol = 0 )

      # calculate median for scaling
      tstat (intable = outtab, \
             column = "mean", \
             outtable = "", \
             lowlim = INDEF,\
             highlim = INDEF )

      # calculate % ADU
      tcalc (table = outtab,
             outcol = "pmean",
             equals = "mean/"//tstat.median)

      # calculate ADU/s
      tcalc (table = outtab,
             outcol = "dc",
             equals = "mean / exptim")
      }

    # delete temporary images and files
    imdelete(images=tmpi, go_ahead=yes, verify=no, >>& "/dev/null")
    delete(timf,yes,verify=no, >>& "/dev/null")
    }

  # graph the result (line plot)
  if ( plot )
    {
    graph(input=tmpl,
          wx1=0.,
          wx2=0.,
          wy1=0.,
          wy2=0.,
          wcs="logical",
          axis=1,
          transpose=no,
          pointmode=no,
          marker="box",
          szmarker=0.005,
          ltypes="",
          colors="",
          logx=no,
          logy=no,
          box=yes,
          ticklabels=yes,
          xlabel="Exposure Time [s]",
          ylabel="Mean ADU",
          xformat="wcsformat",
          yformat="",
          title="INGRID Dark Current",
          lintran=no,
          p1=0.,
          p2=0.,
          q1=0.,
          q2=1.,
          vx1=0.,
          vx2=0.,
          vy1=0.,
          vy2=0.,
          majrx=5,
          minrx=5,
          majry=5,
          minry=5,
          overplot=no,
          append=no,
          device="stdgraph",
          round=no,
          fill=yes)

    # graph the result (overplot markers)
    graph(input=tmpl,
          wx1=0.,
          wx2=0.,
          wy1=0.,
          wy2=0.,
          wcs="logical",
          axis=1,
          transpose=no,
          pointmode=yes,
          marker="box",
          szmarker=0.01,
          ltypes="",
          colors="2",
          logx=no,
          logy=no,
          box=yes,
          ticklabels=yes,
          xlabel="Exposure Time [s]",
          ylabel="Mean ADU",
          xformat="wcsformat",
          yformat="",
          title="INGRID Dark Current",
          lintran=no,
          p1=0.,
          p2=0.,
          q1=0.,
          q2=1.,
          vx1=0.,
          vx2=0.,
          vy1=0.,
          vy2=0.,
          majrx=5,
          minrx=5,
          majry=5,
          minry=5,
          overplot=no,
          append=yes,
          device="stdgraph",
          round=no,
          fill=yes)
    }

  # fit a line to the dark current
  polyfit(input=tmpl,
          order=1,
          weighting=weighting,
          verbose=verbose,
          listdata=no, > fit)

  # fit a line to the dark current and use the output for overplotting
  if ( plot )
    {
    polyfit(input=tmpl,
            order=1,
            weighting=weighting,
            verbose=no,
            listdata=yes) | graph(input="STDIN",
            wx1=0.,
            wx2=0.,
            wy1=0.,
            wy2=0.,
            wcs="logical",
            axis=1,
            transpose=no,
            pointmode=no,
            marker="box",
            szmarker=0.01,
            ltypes="",
            colors="3",
            logx=no,
            logy=no,
            box=yes,
            ticklabels=yes,
            xlabel="Exposure Time [s]",
            ylabel="Mean ADU",
            xformat="wcsformat",
            yformat="",
            title="INGRID Dark Current",
            lintran=no,
            p1=0.,
            p2=0.,
            q1=0.,
            q2=1.,
            vx1=0.,
            vx2=0.,
            vy1=0.,
            vy2=0.,
            majrx=5,
            minrx=5,
            majry=5,
            minry=5,
            overplot=no,
            append=yes,
            device="stdgraph",
            round=no,
            fill=yes)
    }

  # show fit output
  if ( verbose ) {type (fit)}

  # get dark current for output
  list1 = fit
  dark = 0.
  if (fscan (list1, rtmp1, dark) != 2) goto err1
err1:
  darkerr = 0.
  if (fscan (list1, rtmp1, darkerr) != 2) goto err2
err2:

  # output dark current result
  printf("Dark current: %7.3f +- %6.3f ADU/s\n",dark,darkerr)

  # delete temporary images and files
  delete(tmpl,yes,verify=no, >>& "/dev/null")
  delete(fit,yes,verify=no, >>& "/dev/null")

  # clear list variables
  list1 = ""
  list2 = ""
  list3 = ""

end
