# ifindgain - Find conversion gain of detector
# Author : Peter Moore, adapted from ifinddark by Robert Greimel (greimel@ing.iac.es)
# Version: 2. Jan. 2001
##############################################################################
procedure ifindgain (bias1,bias2,flat1,flat2)

string  bias1           {prompt="First bias frame"}
string  bias2           {prompt="Second bias frame"}
string	flat1		{prompt="First flat field"}
string  flat2           {prompt="Second flat field"}
int     box = 2         {prompt="Number of sections in x and y"}
bool	verbose		{prompt="verbose"}

begin
  string	f1, f2, b1, b2, pos, temp_str
  file		tmp_f1, tmp_f2, tmp_b1, tmp_b2, tmpdif, tmpbias
  real          mean, mean1, mean2, stddev, readout_noise, variance, gain
  real          bias_1, bias_2, bias, gain_sum, noise_sum
  int	        xs, xe, ys, ye, xmin, xmax, ymin, ymax, x_step, y_step, step, step_sum  
  
  # copy input arguments to force definition at the beginning of the script
  b1 = bias1
  b2 = bias2
  f1 = flat1
  f2 = flat2
  step = box
  
  # create unique filenames for temporary files
  tmp_f1 = mktemp("tmp$ifindgain")
  tmp_f2 = mktemp("tmp$ifindgain")
  tmp_b1 = mktemp("tmp$ifindgain")
  tmp_b2 = mktemp("tmp$ifindgain")
  tmpdif = mktemp("tmp$ifindgain")
  tmpbias = mktemp("tmp$ifindgain")

  # Process the bias frames
  icheckfile(b1,verbose=no)

  if (!(icheckfile.subtract))
    {
    # calculate post-pre image
    if ( verbose )    print ("calculating post-pre for bias image "//b1//" ...")
    iframediff (input = b1,
                output = tmp_b1,
                subdark = no,
                flatfield = no,
                subsky = no,
                wcs = no,
                verbose = no,
                info = no )
    b1 = tmp_b1
    }
    
  icheckfile(b2,verbose=no)

  if (!(icheckfile.subtract))
    {
    # calculate post-pre image
    if ( verbose )    print ("calculating post-pre for bias image "//b2//" ...")
    iframediff (input = b2,
                output = tmp_b2,
                subdark = no,
                flatfield = no,
                subsky = no,
                wcs = no,
                verbose = no,
                info = no )
    b2 = tmp_b2
    }
    
  # Process the flat fields
  icheckfile(f1,verbose=no)

  # check if data files need to be post-pre subtracted
  if (!(icheckfile.subtract))
    {
    # calculate post-pre image
    if ( verbose )    print ("calculating post-pre for flat field image "//f1//" ...")
    iframediff (input = f1,
                output = tmp_f1,
                subdark = no,
                flatfield = no,
                subsky = no,
                wcs = no,
                verbose = no,
                info = no )
    f1 = tmp_f1
    }

  icheckfile(f2,verbose=no)

  if (!(icheckfile.subtract))
    {
    # calculate post-pre image
    if ( verbose )    print ("calculating post-pre for flat field image "//f2//" ...")
    iframediff (input = f2,
                output = tmp_f2,
                subdark = no,
                flatfield = no,
                subsky = no,
                wcs = no,
                verbose = no,
                info = no )
    f2 = tmp_f2
    }

  # Subtract image files
  imarith(f1, "-", f2, tmpdif)    
  imarith(b1, "-", b2, tmpbias)
  
  # Get the image size parameters
  xmin = 1
  imgets (image=f1, param="i_naxis1")
  xmax = int(imgets.value)
  ymin = 1
  imgets (image=f1, param="i_naxis2")
  ymax = int(imgets.value)
  
  # Compute the step values
  x_step = xmax / step
  y_step = ymax / step
  
  # Zero out the average registers
  gain_sum = 0
  step_sum = 0
  noise_sum = 0

  # Scan through image with a box
  xs = xmin
  xe = xs+x_step - 1
  while (xe <= xmax)
    {
    ys=ymin
    ye=ys+y_step - 1
    while ( ye<= ymax )
      {
      pos =  "["+xs+":"+xe+","+ys+":"+ye+"]"

      # Get mean signal			
      temp_str = f1 + pos
      imstat(temp_str,fields="mean", lower=INDEF, 
	     upper=INDEF, binwidth=0.1, 
	     format=no) | scan(mean1)

      temp_str = f2 + pos
      imstat(temp_str,fields="mean", lower=INDEF, 
	     upper=INDEF, binwidth=0.1, 
	     format=no) | scan(mean2)
	     
      mean = (mean1 + mean2) / 2

      # Get Variance
      temp_str = tmpdif + pos			
      imstat(temp_str,fields="stddev", lower=INDEF, 
	     upper=INDEF, binwidth=0.1, format=no) | scan(stddev)

      # Get Bias and Noise
      temp_str = b1 + pos
      imstat(temp_str,fields="mean", lower=INDEF, 
             upper=INDEF, binwidth=0.1, format=no) | scan(bias_1)

      temp_str = b2 + pos
      imstat(temp_str,fields="mean", lower=INDEF, 
             upper=INDEF, binwidth=0.1, format=no) | scan(bias_2)
             
      bias = (bias_1 + bias_2) / 2

      temp_str = tmpbias + pos			
      imstat(temp_str,fields="stddev", lower=INDEF, 
             upper=INDEF, binwidth=0.1, format=no) | scan(readout_noise)
					
      # Calculate the gain
      variance = stddev - readout_noise
      gain = (mean - bias) / (variance * variance)

      if ( verbose )  
        {  
        printf("section=%-21s mean=%-7.2f var=%-5.2f ron=%-5.2f gain=%-4.2f bias=%-6.2f\n",pos, mean, variance, (readout_noise * 0.5), gain, bias)
        gain_sum = gain_sum + gain
        noise_sum = noise_sum + (readout_noise * 0.5)
        step_sum = step_sum + 1
        }
      else
        {
        print(mean," ",variance," ",gain)
        } 

      ys = ye + 1
      ye = ys + y_step - 1
    }
    xs = xe + 1
    xe = xs + x_step - 1
  }

  if ( verbose )
    {
    gain = gain_sum / step_sum
    readout_noise = (noise_sum / step_sum) * gain
    printf("\nAverage conversion factor is %-4.2f e-/ADU, readout noise is %-5.2f e- rms.\n",gain, readout_noise)
    }
    
  # delete temporary image files
  imdel(images=tmp_f1, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmp_f2, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmp_b1, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmp_b2, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmpdif, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmpbias, go_ahead=yes, verify=no, >>& "/dev/null")

end
