# ifindphot - Plot photon transfer curve.
# Author : Peter Moore, adapted from ifinddark by Robert Greimel (greimel@ing.iac.es)
# Version: 4. Jan. 2001
##############################################################################
procedure ifindphot(input)
string  bias1           {prompt="First bias frame"}
string  bias2           {prompt="Second bias frame"}
string	input		{prompt="List of flat field frames"}
real    clip = 3	{prompt="clipping factor in sigma"}
bool	verbose		{prompt="verbose"}

string *list1	{prompt="Ignore this parameter(list1)"}

begin
  string	ini, b1, b2
  string	f1, f2, fs1, fs2, plot_title
  file		tmp_b1, tmp_b2, tmp_bias, tmp1, tmp2, tmpv, tmpl, datf, datg
  real		etime, mean, mean1, mean2, middle, bias, stddev, s, gain, ron, var, gain_sum, step_sum

  # copy input arguments to force definition at the beginning of the script
  b1 = bias1
  b2 = bias2
  ini = input
  plot_title = ""

  # create unique filenames for temporary files
  tmp_b1 = mktemp("tmp$ifindphot")
  tmp_b2 = mktemp("tmp$ifindphot")
  tmp_bias = mktemp("tmp$ifindphot")
  tmp1 = mktemp("tmp$ifindphot")
  tmp2 = mktemp("tmp$ifindphot")
  tmpl = mktemp("tmp$ifindphot")
  tmpv = mktemp("tmp$ifindphot")
  datf = mktemp("tmp$ifindphot")
  datg = mktemp("tmp$ifindphot")

  # Process the bias frames
  icheckfile(b1,verbose=no)

  if (!(icheckfile.subtract))
    {
    # calculate post-pre image
    if ( verbose )    print ("calculating post-pre for bias image "//b1//" ...")
    iframediff (input = b1,
                output = tmp_b1,
                subdark = no,
                flatfield = no,
                subsky = no,
                wcs = no,
                verbose = no,
                info = no )
    b1 = tmp_b1
    }
    
  icheckfile(b2,verbose=no)

  if (!(icheckfile.subtract))
    {
    # calculate post-pre image
    if ( verbose )    print ("calculating post-pre for bias image "//b2//" ...")
    iframediff (input = b2,
                output = tmp_b2,
                subdark = no,
                flatfield = no,
                subsky = no,
                wcs = no,
                verbose = no,
                info = no )
    b2 = tmp_b2
    }

  # Get variance of the bias frames
  imarith(b1, "-", b2, tmp_bias)
 
  # Get Bias and Noise from bias frames
  imstat(tmp_b1,fields="midpt,stddev",lower=INDEF,
         upper=INDEF,binwidth=0.1,format=no) | scanf("%f %f",middle,s)

  imstat(tmp_b1,fields="mean", lower=(middle - clip*s), 
         upper=(middle + clip*s), binwidth=0.1, format=no) | scan(mean1)

  imstat(tmp_b2,fields="midpt,stddev",lower=INDEF,
         upper=INDEF,binwidth=0.1,format=no) | scanf("%f %f",middle,s)

  imstat(tmp_b2,fields="mean", lower=(middle - clip*s), 
         upper=(middle + clip*s), binwidth=0.1, format=no) | scan(mean2)

  imstat(tmp_bias,fields="midpt,stddev",lower=INDEF,
         upper=INDEF,binwidth=0.1,format=no) | scanf("%f %f",middle,s)

  imstat(tmp_bias,fields="stddev", lower=(middle - clip*s), 
         upper=(middle + clip*s), binwidth=0.1, format=no) | scan(ron)
                 
  bias = (mean1 + mean2) / 2

  if (verbose) printf("Mean of bias frames = %-5.2f ADU, RON = %-5.2f ADU rms.\n", bias, (ron / 2))
   
  # expand the flat field list.
  sections(ini,option="fullname",> tmpl)
 
  # zero out the average registers
  gain_sum = 0
  step_sum = 0
  
  # Work through list of input files
  list1 = tmpl
    
  while (fscan(list1, f1) != EOF)
    {
    if (fscan(list1, f2) != EOF)
      { 

      # find exposure times
      ifileinfo(input=f1, verbose=no)
      etime = ifileinfo.exptim
 
      # Check that integration times are the same
      ifileinfo(input=f2, verbose=no)
      if (ifileinfo.exptim == etime)
        {
        # check file subtraction and format
        icheckfile(f1,verbose=no)

        # check if first file needs to be post-pre subtracted
        if ( icheckfile.subtract )
          {
          fs1 = f1
          }
        else
          {
          # delete previous temporary image
          imdel(images=tmp1, go_ahead=yes, verify=no, >>& "/dev/null")

          # calculate post-pre image
          iframediff (input = f1,
                      output = tmp1,
                      subdark = no,
                      flatfield = no,
                      subsky = no,
                      wcs = no,
                      verbose = no,
                      info = no )
          fs1 = tmp1
          }

        # check file subtraction and format
        icheckfile(f2,verbose=no)

        # check if second file needs to be post-pre subtracted
        if ( icheckfile.subtract )
          {
          fs2 = f2
          }
        else
          {
          # delete previous temporary image
          imdel(images=tmp2, go_ahead=yes, verify=no, >>& "/dev/null")

          # calculate post-pre image
          iframediff (input = f2,
                      output = tmp2,
                      subdark = no,
                      flatfield = no,
                      subsky = no,
                      wcs = no,
                      verbose = no,
                      info = no )
          fs2 = tmp2
          }
          
        # calculate statistics
        imstatistics (fs1,
                      fields="midpt,stddev",
                      lower=INDEF,
                      upper=INDEF,
                      binwidth=0.1,
                      format=no) | scanf("%f %f",middle,s)

        # re-calculate statistics with sigma clipping
        imstatistics (fs1,
                      fields="mean",
                      lower=(middle - clip*s),
                      upper=(middle + clip*s),
                      binwidth=0.1,
                      format=no) | scanf("%f",mean1)

        # Same for second flat field
        imstatistics (fs2,
                      fields="midpt,stddev",
                      lower=INDEF,
                      upper=INDEF,
                      binwidth=0.1,
                      format=no) | scanf("%f %f",middle,s)

        imstatistics (fs2,
                      fields="mean",
                      lower=(middle - clip*s),
                      upper=(middle + clip*s),
                      binwidth=0.1,
                      format=no) | scanf("%f",mean2)
                        
        # Check that means are within 1 percent
        mean = (mean1 + mean2) / 2.0
        if (((mean1 - mean2) / mean) <= 0.01)
          {

          # subtract flat fields to obtain deviation
          imdel(images=tmpv, go_ahead=yes, verify=no, >>& "/dev/null")
          imarith( fs1, "-", fs2, tmpv)

          imstat(tmpv,fields="midpt,stddev",lower=INDEF,
                 upper=INDEF,binwidth=0.1,format=no) | scanf("%f %f",middle,s)

          imstat(tmpv,fields="stddev",lower=(middle - clip*s), 
                 upper=(middle + clip*s),binwidth=0.1,format=no) | scanf("%f",stddev)
  
          mean = mean - bias
          var = stddev - ron
          gain = mean / (var * var)
          gain_sum = gain_sum + gain
          step_sum = step_sum + 1
          
          # write mean and ADU / sec to temporary file
          print(mean//" "//var, >> datf)
          print(mean//" "//gain, >> datg)
          
          # print verbose information
          if ( verbose ) 
            {
            printf("Files %s/%s : Mean = %-7.2f Variance = %-5.2f Gain = %-5.2f\n",f1, f2, mean, var, gain)
            }
          }
        else
          {
          print ("Flat pair trashed ... Mean values of "//f1//" ("//mean1//") and "//f2//" ("//mean2//") differ by greater than 1%.")
          }
        }
      else
        {
        print ("Flat pair trashed ... Integration times for "//f1//" ("//etime//") and "//f2//" ("//ifileinfo.exptim//") are different")
        }          
      }
    }

  # sort mean values into order
  del(tmpl,yes,verify=no)
  sort(input_file = datf,
       column = 0,
       ignore_white = no,
       numeric_sort = yes,
       reverse_sort = no,
       > tmpl)

  # calculate the mean gain value
  gain = gain_sum / step_sum
    
  del(datf,yes,verify=no)
  list1 = tmpl

  # calculate the mean of the gain
  while(fscan(list1,mean,stddev) != EOF)
  {
    mean = mean * gain
    var = stddev * gain
    print(mean//" "//(var * var)//" "//mean//" "//sqrt(mean), >> datf)
  }
   
  # Setup a graph title string
  fprint(plot_title, "ifindphot - Photon transfer Curve - Average gain = %-5.2f e-/ADU ron = %-5.2f e- rms", gain, (ron /2 * gain))

  # graph line plot of mean / variance
  graph(input=datf,
        wx1=0.,
        wx2=0.,
        wy1=0.,
        wy2=0.,
        wcs="logical",
        axis=1,
        transpose=no,
        pointmode=no,
        marker="box",
        szmarker=0.005,
        ltypes="",
        colors="",
        logx=no,
        logy=no,
        box=yes,
        ticklabels=yes,
        xlabel="Mean Signal e-",
        ylabel="Variance Squared e",
        xformat="wcsformat",
        yformat="",
        title=plot_title,
        lintran=no,
        p1=0.,
        p2=0.,
        q1=0.,
        q2=1.,
        vx1=0.,
        vx2=0.,
        vy1=0.,
        vy2=0.,
        majrx=5,
        minrx=5,
        majry=5,
        minry=5,
        overplot=no,
        append=no,
        device="stdgraph",
        round=no,
        fill=yes)
 
 # graph markers of mean / variance
  graph(input=datf,
        wx1=0.,
        wx2=0.,
        wy1=0.,
        wy2=0.,
        wcs="logical",
        axis=1,
        transpose=no,
        pointmode=yes,
        marker="circle",
        szmarker=0.01,
        ltypes="",
        colors="2",
        logx=no,
        logy=no,
        box=yes,
        ticklabels=yes,
        xlabel="Mean Signal e-",
        ylabel="Variance Squared e",
        xformat="wcsformat",
        yformat="",
        title=plot_title,
        lintran=no,
        p1=0.,
        p2=0.,
        q1=0.,
        q2=1.,
        vx1=0.,
        vx2=0.,
        vy1=0.,
        vy2=0.,
        majrx=5,
        minrx=5,
        majry=5,
        minry=5,
        overplot=no,
        append=yes,
        device="stdgraph",
        round=no,
        fill=yes)

  # delete temporary images and files
  imdel(images=tmp_b1, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmp_b2, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmp_bias, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmp1, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmp2, go_ahead=yes, verify=no, >>& "/dev/null")
  imdel(images=tmpv, go_ahead=yes, verify=no, >>& "/dev/null")
  del(tmpl,yes,verify=no, >>& "/dev/null")
  del(datf,yes,verify=no, >>& "/dev/null")
  del(datg,yes,verify=no, >>& "/dev/null")

  # clear list variables
  list1 = ""
  
end
