# ilinearity - calculates linearity of detector
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 06. Sep. 2001
# History: Based on a midas script by Thomas Augusteijn
##############################################################################
procedure ilinearity (files)

string	files			{prompt="List of data files"}

string	outtab			{prompt="Root Name of output tables"}

string	section = "[*,*]"	{prompt="Selected image section"}
int	xdiv			{prompt="number of divisions in x"}
int	ydiv			{prompt="number of divisions in y"}

real	clip = 3		{prompt="clipping factor in sigma"}

string	center = "mean"		{prompt="Central statistical measure",
				    enum="mean|midpt|mode"}
real	binwidth = 0.1		{prompt="Bin width of histogram in sigma"}

real	timerr			{prompt="Timing Error [s]"}

bool	plot = yes		{prompt="plot the result ?"}
bool	verbose = yes		{prompt="Verbose output?"}

string	*list1
string	*list2

begin
  string	darkimg,infiles,trim,name,name2
  string	table,tabd1,tabd2,tabr1,tabr2,tmptab
  string	ptable,ptabd1,ptabd2,ylabel
  string	stats,stats1,stats2,stmp1,stmp2,filsec
  string	infs,indark1,indark2,indata,dim,im1,im2
  string	ut1,ut2,key,cmd,lastkey,col
  string	outroot,outext
  int		ip,xs,xe,xdp,ys,ye,ydp,i,j,mndr,coadd,gwcs
  real		low,high,ron,ronerr,avg,avgerr,rtmp1,rtmp2,md1,md2
  real		med1,med2,exp1,exp2,expd,stdev,var
  real		gx,gy
  bool		fdark,point,percent,replot,pavg,line

  # remove any temporary files that could have been left around by a previous
  # aborted run
  delete("tmp$ilinearity*",yes,verify=no,>>& "/dev/null")

  # need to copy arguments to define them for sure
  infiles = files

  # define variables
  point = yes
  line = yes
  pavg = yes
  percent = no

  # generate temporary table file names
  table = mktemp ("tmp$ilinearity")
  tabd1 = table//"_d1.tab"
  tabd2 = table//"_d2.tab"
  tabr1 = table//"_r1.tab"
  tabr2 = table//"_r2.tab"
  table = table//".tab"

  # check if output table already exists
  itabaccess(table=outtab,verbose=no)
  if ( itabaccess.exists ) outtab = itabaccess.outname
  ip = stridx( ".",outtab )
  if ( itabaccess.exists && ip == 0 ) ip = strlen(outtab)+1
  if ( ip > 0 )
    {
    outroot = substr(outtab,1,ip-1)
    outext = substr(outtab,ip+1,strlen(outtab))
    }
  else
    {
    outroot = outtab
    outext = "fit"
    }
  #
  if ( itabaccess.exists )
    {
    beep
    print("")
    print("WARNING: output table "//outtab//" does already exist")
    print("         Plotting results from existing table ...")
    tcopy(intable=outroot//"."//outext,outtable=table,verbose=no)
    tcopy(intable=outroot//"_d1."//outext,outtable=tabd1,verbose=no)
    tcopy(intable=outroot//"_d2."//outext,outtable=tabd2,verbose=no)
    tcopy(intable=outroot//"_r1."//outext,outtable=tabr1,verbose=no)
    tcopy(intable=outroot//"_r2."//outext,outtable=tabr2,verbose=no)
    beep
    goto plot
    }

  # expand image name.
  infs = mktemp ("tmp$ilinearity")
  sections(infiles,option="fullname",> infs)

  # loop over files and split into categories
  # This requires that the user lists all the darks taken before the
  # linearity test first, then the flats for the linearity test and
  # last the darks taken after the linearity test.
  # These restrictions on the order of the input files could be relaxed
  # if a table was used to sort out exposures ...
  indark1 = mktemp ("tmp$ilinearity")
  indark2 = mktemp ("tmp$ilinearity")
  indata = mktemp ("tmp$ilinearity")
  list1 = infs
  ip = 1
  while ( fscan(list1, name) != EOF )
    {
    imgets(image=name//"[0]", param="OBSTYPE")
    if ( imgets.value == "DARK" )
      {
      if ( ip == 1 ) print(name,>>indark1)
      if ( ip == 2 ) print(name,>>indark2)
      next
      }
    # ImpB uses RUN, UltraDAS uses TARGET ...
    if ( imgets.value == "RUN" || imgets.value == "TARGET")
      {
      print(name,>>indata)
      ip = 2
      next
      }
    print("WARNING: frame "//name//" is of unknown or unused type "//imgets.value)
    }

  # find the pair of darks with the smallest exposure time from the
  # list of darks taken before the linearity test.
  # still TBD
  i = 1
  list1 = indark1
  while ( fscan(list1, name) != EOF )
    {
    if ( i == 1 ) stmp1=name
    if ( i == 2 ) stmp2=name
    i = i+1
    }

  # find read noise for first series of dark images
  print("Calculating read noise before linearity test ...")
  ifindron(dark1=stmp1,
           dark2=stmp2,
           outtab=tabr1,
           section=section,
           xdiv=xdiv,
           ydiv=ydiv,
           clip=clip,
           center=center,
           binwidth=binwidth,
           plot=no,
           verbose=no)

  # find information for first series of dark images
  print("Calculating dark current before linearity test ...")
  ifinddark(input="@"//indark1,
            outtab=tabd1,
            section=section,
            xdiv=xdiv,
            ydiv=ydiv,
            clip=clip,
            center=center,
            weighting="instrumental",
            plot=no,
            verbose=no)

  # find the pair of darks with the smallest exposure time from the
  # list of darks taken after the linearity test.
  # still TBD
  i = 1
  list1 = indark2
  while ( fscan(list1, name) != EOF )
    {
    if ( i == 1 ) stmp1=name
    if ( i == 2 ) stmp2=name
    i = i+1
    }

  # find read noise for second series of dark images
  print("Calculating read noise after linearity test ...")
  ifindron(dark1=stmp1,
           dark2=stmp2,
           outtab=tabr2,
           section=section,
           xdiv=xdiv,
           ydiv=ydiv,
           clip=clip,
           center=center,
           binwidth=binwidth,
           plot=no,
           verbose=no)

  # find information for second series of dark images
  print("Calculating dark current after linearity test ...")
  ifinddark(input="@"//indark2,
            outtab=tabd2,
            section=section,
            xdiv=xdiv,
            ydiv=ydiv,
            clip=clip,
            center=center,
            weighting="instrumental",
            plot=no,
            verbose=no)

#  ifileinfo(input=indark1, verbose=no)
#  rtmp1 = ifileinfo.exptim
#  coadd = ifileinfo.coadd
#  mndr = ifileinfo.mndr

  # find exposure information for second dark image
#  ifileinfo(input=indark2, verbose=no)

  # check if exposure times are the same. If not, print a warning
#  if ( ifileinfo.exptim != rtmp1 )
#    {
#    print(" ")
#    print("WARNING: exposure times for frames is different")
#    print(" ")
#    print("         first  dark: "//rtmp1)
#    print("         second dark: "//ifileinfo.exptim)
#    print(" ")
#    }

  # check that number of coadds are the same
#  if ( ifileinfo.coadd != coadd )
#    {
#    beep
#    print("")
#    print("ERROR: number of coadds for frames is different")
#    print("")
#    print("         first  dark: "//coadd)
#    print("         second dark: "//ifileinfo.coadd)
#    print("")
#    print("ilinearity aborted")
#    beep
#    bye
#    }

  # check that number of coadds is > 0
#  if ( coadd < 1 )
#    {
#    print(" ")
#    print("WARNING: number of coadds is "//coadd//" - should be >= 1")
#    print("         adjusting to 1")
#    print(" ")
#    coadd = 1
#    }

  # check that number of MNDR are the same
#  if ( ifileinfo.mndr != mndr )
#    {
#    beep
#    print("")
#    print("ERROR: number of MNDR for frames is different")
#    print("")
#    print("         first  dark: "//mndr)
#    print("         second dark: "//ifileinfo.mndr)
#    print("")
#    print("ilinearity aborted")
#    beep
#    bye
#    }

  # check that number of MNDR is 1
#  if ( mndr != 1 )
#    {
#    print(" ")
#    print("WARNING: number of MNDR is "//mndr//" - should be 1")
#    printf("         The result will be wrong (about %5.2f to small) !",sqrt(mndr))
#    print(" ")
#    }

  # create temporary file to hold list of image sections to find
  # statistics for
  im1 = mktemp ("tmp$ilinearity")
  im2 = mktemp ("tmp$ilinearity")
  dim = mktemp ("tmp$ilinearity")
  stats = mktemp ("tmp$ilinearity")
  filsec = mktemp ("tmp$ifinddark")
  stats1 = mktemp ("tmp$ifinddark")
  stats2 = mktemp ("tmp$ifinddark")

  # now loop over the data files
  list1 = indata
  while ( fscan(list1, name) != EOF )
    {
    # this assumes we always get images in pairs ...
    # will need further checking.
    ip=fscan(list1, name2)

    print("working on image pair "//name//" and "//name2)

    # find exposure time information for flats
    ifileinfo(input=name, verbose=no)
    exp1 = ifileinfo.exptim
    ifileinfo(input=name2, verbose=no)
    exp2 = ifileinfo.exptim

    # need check on exp1 == exp2 here

    # find the dark current frame with the same exposure time that is
    # closest in time to the exposure
    darkimg = ""
    fdark = no
    list2 = indark1
    while ( !fdark && fscan(list2, stmp1) != EOF )
      {
      ifileinfo(input=stmp1, verbose=no)
      expd = ifileinfo.exptim
      if ( abs(exp1 - expd) < 0.2 )
        {
	fdark = yes
	darkimg = stmp1
	}
      }

    # need check on fdark here !
    print ("subtracting dark image "//darkimg)

    isubtract (inputr1 = name,
               inputr2 = darkimg,
               output = im1,
               verbose = no,
               info = no)

    isubtract (inputr1 = name2,
               inputr2 = darkimg,
               output = im2,
               verbose = no,
               info = no)

    # find UT
    imgets(image=im1, param="UTSTART")
    ut1 = imgets.value
    imgets(image=im2, param="UTSTART")
    ut2 = imgets.value

    # find median
    isectionstat(input=im1,
                 outtab="",
                 field="midpt",
                 section=section,
                 xdiv=xdiv,
                 ydiv=ydiv,
                 clip=clip,
                 center=center,
                 plot=no,
                 verbose=no)
    med1=isectionstat.value

    isectionstat(input=im2,
                 outtab="",
                 field="midpt",
                 section=section,
                 xdiv=xdiv,
                 ydiv=ydiv,
                 clip=clip,
                 center=center,
                 plot=no,
                 verbose=no)
    med2=isectionstat.value

    # subtract the two images
    imarith (im1, "-", im2, dim,
             title="",
             divzero=0., hparams="", pixtype="", calctype="",
             verbose=no, noact=no)

    # find the standard deviation
    isectionstat(input=dim,
                 outtab="",
                 field="stddev",
                 section=section,
                 xdiv=xdiv,
                 ydiv=ydiv,
                 clip=clip,
                 center=center,
                 plot=no,
                 verbose=no)
    stdev=isectionstat.value

    print(name//" "//name2//" "//darkimg//" "//ut1//" "//ut2//" ",exp1,exp2,med1,med2,stdev, >> stats)

    # delete the temporary images
    imdelete(im1,yes,verify=no)
    imdelete(im2,yes,verify=no)
    imdelete(dim,yes,verify=no)
    }

  # read statistics into table
  tcreate (table = table, \
           cdfile = "ingrid_ql$ilinearity.cd", \
           datafile = stats, \
           uparfile = "", \
           nskip = 0, \
           nlines = 0, \
           nrows = 0, \
           hist = no, \
           extrapar = 0, \
           tbltype = "default", \
           extracol = 0 )

  # sort the table by exposure time
  tsort (table = table,
         columns = "exp1",
         ascend = yes,
         casesens = no)

  # calculate variance
  tcalc (table = table,
         outcol = "var",
         equals = "stdev*stdev/2")

  # calculate gain [e/ADU]
  tcalc (table = table,
         outcol = "gain1",
         equals = "median1/var")

  tcalc (table = table,
         outcol = "gain2",
         equals = "median2/var")

  # calculate Flux [ADU/s]
  tcalc (table = table,
         outcol = "flux1",
         equals = "median1/exp1")

  tcalc (table = table,
         outcol = "flux2",
         equals = "median2/exp2")

  #####
  # all calculations are done at this point, so let's save the output
  # tables if requested. We should do this as early as possible to
  # prevent costly recalculation in case of program crashes further down.
  #####
  if ( outtab != "" )
    {
    if (verbose) print("saving tables to "//outtab)
    tcopy(intable=table,outtable=outroot//"."//outext,verbose=no)
    tcopy(intable=tabd1,outtable=outroot//"_d1."//outext,verbose=no)
    tcopy(intable=tabd2,outtable=outroot//"_d2."//outext,verbose=no)
    tcopy(intable=tabr1,outtable=outroot//"_r1."//outext,verbose=no)
    tcopy(intable=tabr2,outtable=outroot//"_r2."//outext,verbose=no)
    }

plot:
  #####
  # correct for timing error
  #####
  if ( timerr != 0. )
    {
    # adjust exposure times
    tcalc (table = table,
           outcol = "exp1",
           equals = "exp1 + "//timerr)
    tcalc (table = table,
           outcol = "exp2",
           equals = "exp2 + "//timerr)
    # recalculate flux
    tcalc (table = table,
           outcol = "flux1",
           equals = "median1/exp1")
    tcalc (table = table,
           outcol = "flux2",
           equals = "median2/exp2")
    # adjust exposure times
    tcalc (table = tabd1,
           outcol = "exptim",
           equals = "exptim + "//timerr)
    # recalculate flux
    tcalc (table = tabd1,
           outcol = "dc",
           equals = "mean/exptim")
    # adjust exposure times
    tcalc (table = tabd2,
           outcol = "exptim",
           equals = "exptim + "//timerr)
    # recalculate flux
    tcalc (table = tabd2,
           outcol = "dc",
           equals = "mean/exptim")
    }

  #####
  # generate percentage tables
  #####
  # generate temporary ptable file names
  ptable = mktemp ("tmp$ilinearity")
  ptabd1 = ptable//"_d1.fit"
  ptabd2 = ptable//"_d2.fit"
  ptable = ptable//".fit"

  # percentage table
  tcopy(intable=table,outtable=ptable,verbose=no)
  for (i=1;i<=4;i=i+1)
    {
    if ( i == 1 ) col="median1"
    if ( i == 2 ) col="median2"
    if ( i == 3 ) col="flux1"
    if ( i == 4 ) col="flux2"

    # calculate column statistics
    tstat (intable = ptable, \
           column = col, \
           outtable = "", \
           lowlim = INDEF,\
           highlim = INDEF )

    # calculate %
    tcalc (table = ptable,
           outcol = col,
           equals = "(100*"//col//"/"//tstat.median//")-100")
    }

  tcopy(intable=tabd1,outtable=ptabd1,verbose=no)
  for (i=1;i<=2;i=i+1)
    {
    if ( i == 1 ) col="mean"
    if ( i == 2 ) col="dc"

    # calculate column statistics
    tstat (intable = ptabd1, \
           column = col, \
           outtable = "", \
           lowlim = INDEF,\
           highlim = INDEF )
    # calculate %
    tcalc (table = ptabd1,
           outcol = col,
           equals = "(100*"//col//"/"//tstat.median//")-100")
    }

  tcopy(intable=tabd2,outtable=ptabd2,verbose=no)
  for (i=1;i<=2;i=i+1)
    {
    if ( i == 1 ) col="mean"
    if ( i == 2 ) col="dc"

    # calculate column statistics
    tstat (intable = ptabd2, \
           column = col, \
           outtable = "", \
           lowlim = INDEF,\
           highlim = INDEF )
    # calculate %
    tcalc (table = ptabd2,
           outcol = col,
           equals = "(100*"//col//"/"//tstat.median//")-100")
    }

  #####
  # plot results if requested
  #####
  if ( plot )
    {
    # some general plot parameter settings
    pltpar.szmarker = 0.01
    # set key to generate initial plot
    key = "a"
    replot = no
    while ( key != "q" )
      {
      switch ( key )
        {
        ###
        # plot ADU vs. exposure time
        ###
        case 'a':
          {
          # save key
          lastkey = key
          # check if percentages should be plotted
          if ( percent )
            {
            tmptab = ptable
            ylabel = "% Median"
            }
          else
            {
            tmptab = table
            ylabel = "Median [ADU]"
            }
          # plot graph
          itabplot(itable1 = tmptab,
                   col1 = "exp1 median1",
                   itable2 = tmptab,
                   col2 = "exp2 median2",
                   title = "INGRID Linearity (TC="//timerr//")",
                   xlabel = "Exposure Time [s]",
                   ylabel = ylabel,
                   point = point,
                   line = line,
                   pavg = pavg,
                   xerr = no,
                   yerr = yes)
          }
        ###
        # plot ADU vs. exposure time for both dark currents
        ###
        case 'd':
          {
          # save key
          lastkey = key
          # check if percentages should be plotted
          if ( percent )
            {
            # plot graph
            itabplot(itable1 = ptabd1,
                     col1 = "exptim mean",
                     itable2 = ptabd2,
                     col2 = "exptim mean",
                     title = "INGRID Dark Current (TC="//timerr//")",
                     xlabel = "Exposure Time [s]",
                     ylabel = "% ADU",
                     point = point,
                     line = line,
                     pavg = pavg,
                     xerr = no,
                     yerr = yes)
            }
          else
            {
            # plot graph
            itabplot(itable1 = tabd1,
                     col1 = "exptim mean",
                     itable2 = tabd2,
                     col2 = "exptim mean",
                     title = "INGRID Dark Current (TC="//timerr//")",
                     xlabel = "Exposure Time [s]",
                     ylabel = "ADU",
                     point = point,
                     line = line,
                     pavg = pavg,
                     xerr = no,
                     yerr = yes)
            }
          }
        ###
        # plot ADU/s vs. exposure time for both dark currents
        ###
        case 'e':
          {
          # save key
          lastkey = key
          # check if percentages should be plotted
          if ( percent )
            {
            # plot graph
            itabplot(itable1 = ptabd1,
                     col1 = "exptim dc",
                     itable2 = ptabd2,
                     col2 = "exptim dc",
                     title = "INGRID Dark Current (TC="//timerr//")",
                     xlabel = "Exposure Time [s]",
                     ylabel = "% ADU/s",
                     point = point,
                     line = line,
                     pavg = pavg,
                     xerr = no,
                     yerr = yes)
            }
          else
            {
            # plot graph
            itabplot(itable1 = tabd1,
                     col1 = "exptim dc",
                     itable2 = tabd2,
                     col2 = "exptim dc",
                     title = "INGRID Dark Current (TC="//timerr//")",
                     xlabel = "Exposure Time [s]",
                     ylabel = "ADU / s",
                     point = point,
                     line = line,
                     pavg = pavg,
                     xerr = no,
                     yerr = yes)
            }
          }
        ###
        # plot Flux vs. ADU
        ###
        case 'f':
          {
          # save key
          lastkey = key
          # check if percentages should be plotted
          if ( percent )
            {
            tmptab = ptable
            ylabel = "% Flux"
            }
          else
            {
            tmptab = table
            ylabel = "Flux [ADU/s]"
            }
          # plot graph
          itabplot(itable1 = tmptab,
                   col1 = "median1 flux1",
                   itable2 = tmptab,
                   col2 = "median2 flux2",
                   title = "INGRID Linearity (TC="//timerr//")",
                   xlabel = "Median [ADU]",
                   ylabel = ylabel,
                   point = point,
                   line = line,
                   pavg = pavg,
                   xerr = yes,
                   yerr = yes)
          }
        ###
        # plot Flux vs. Exp
        ###
        case 'h':
          {
          # save key
          lastkey = key
          # check if percentages should be plotted
          if ( percent )
            {
            tmptab = ptable
            ylabel = "% Flux"
            }
          else
            {
            tmptab = table
            ylabel = "Flux [ADU/s]"
            }
          # plot graph
          itabplot(itable1 = tmptab,
                   col1 = "exp1 flux1",
                   itable2 = tmptab,
                   col2 = "exp2 flux2",
                   title = "INGRID Linearity (TC="//timerr//")",
                   xlabel = "Exposure Time [s]",
                   ylabel = ylabel,
                   point = point,
                   line = line,
                   pavg = pavg,
                   xerr = no,
                   yerr = yes)
          }
        ###
        # toggle line plotting mode
        ###
        case 'l':
          {
          line = !line
          #print("Line mode: "//line)
          if ( !point && !line ) point = yes
          replot = yes
          }
        ###
        # quit
        ###
        case 'q':
          {
          goto ende
          }
        ###
        # plot both read noise statistics
        ###
        case 'r':
          {
          # save key
          lastkey = key
          # plot graph
          axispar.xlabel = "Standard Deviation"
          axispar.ylabel = "Number"
          axispar.title = "INGRID Read Noise"
          # plot dark current before
          pltpar.crvcolor = 3
          thistogram (tabr1,"STDOUT","stdev") | sgraph (crvstyle="pseudohist")
          dvpar.append = yes
          # plot dark current after
          pltpar.crvcolor = 4
          thistogram (tabr2,"STDOUT","stdev") | sgraph (crvstyle="pseudohist")
          }
        ###
        # plot Flux vs. UT
        ###
        case 't':
          {
          # save key
          lastkey = key
          # check if percentages should be plotted
          if ( percent )
            {
            tmptab = ptable
            ylabel = "% Flux"
            }
          else
            {
            tmptab = table
            ylabel = "Flux [ADU/s]"
            }
          # plot graph
          itabplot(itable1 = tmptab,
                   col1 = "ut1 flux1",
                   itable2 = tmptab,
                   col2 = "ut2 flux2",
                   title = "INGRID Linearity (TC="//timerr//")",
                   xlabel = "UT",
                   ylabel = ylabel,
                   point = yes,
                   line = no,
                   pavg = no,
                   xerr = no,
                   yerr = no)
          }
        ###
        # plot Exp vs. UT
        ###
        case 'u':
          {
          # save key
          lastkey = key
          # plot graph
          itabplot(itable1 = table,
                   col1 = "ut1 exp1",
                   itable2 = table,
                   col2 = "ut2 exp2",
                   title = "INGRID Linearity (TC="//timerr//")",
                   xlabel = "UT",
                   ylabel = "Exposure Time [s]",
                   point = yes,
                   line = no,
                   pavg = no,
                   xerr = no,
                   yerr = no)
          }
        ###
        # plot Variance vs. ADU
        ###
        case 'v':
          {
          # save key
          lastkey = key
          # plot graph
          itabplot(itable1 = table,
                   col1 = "median1 var",
                   itable2 = table,
                   col2 = "median2 var",
                   title = "INGRID Linearity",
                   xlabel = "Median [ADU]",
                   ylabel = "Variance",
                   point = point,
                   line = line,
                   pavg = pavg,
                   xerr = yes,
                   yerr = no)
          }
        ###
        # plot gain vs. ADU
        ###
        case 'x':
          {
          # save key
          lastkey = key
          # plot graph
          axispar.xlabel = "ADU"
          axispar.ylabel = "Gain"
          axispar.title = "INGRID Gain"
          # plot graph
          itabplot(itable1 = table,
                   col1 = "median1 gain1",
                   itable2 = table,
                   col2 = "median2 gain2",
                   title = "INGRID Gain",
                   xlabel = "Median [ADU]",
                   ylabel = "Gain",
                   point = point,
                   line = line,
                   pavg = pavg,
                   xerr = yes,
                   yerr = yes)
          }
        ###
        # plot Median vs. UT
        ###
        case 'y':
          {
          # save key
          lastkey = key
          # check if percentages should be plotted
          if ( percent )
            {
            tmptab = ptable
            ylabel = "% Median"
            }
          else
            {
            tmptab = table
            ylabel = "Median [ADU]"
            }
          # plot graph
          itabplot(itable1 = tmptab,
                   col1 = "ut1 median1",
                   itable2 = tmptab,
                   col2 = "ut2 median2",
                   title = "INGRID Linearity",
                   xlabel = "UT",
                   ylabel = ylabel,
                   point = yes,
                   line = no,
                   pavg = no,
                   xerr = no,
                   yerr = yes)
          }
        ###
        # plot gain vs. exposure time
        ###
        case 'z':
          {
          # save key
          lastkey = key
          # plot graph
          itabplot(itable1 = table,
                   col1 = "exp1 gain1",
                   itable2 = table,
                   col2 = "exp2 gain2",
                   title = "INGRID Gain",
                   xlabel = "Exposure Time [s]",
                   ylabel = "Gain",
                   point = point,
                   line = line,
                   pavg = pavg,
                   xerr = yes,
                   yerr = yes)
          }
        ###
        # toggle point plotting mode
        ###
        case '.':
          {
          point = !point
          #print("Point mode: "//point)
          if ( !point && !line ) line = yes
          replot = yes
          }
        ###
        # toggle percentage display mode
        ###
        case '%':
          {
          percent = !percent
          #print("Percent mode: "//percent)
          replot = yes
          }
        ###
        # toggle average display mode
        ###
        case '@':
          {
          pavg = !pavg
          #print("Plot average mode: "//pavg)
          replot = yes
          }
        ###
        # print the help for available cursor commands
        ###
#                           CURSOR KEY COMMAND SUMMARY
#
#? Help              g Graphics cursor   n Next frame        u Vector plot
#? Help
#a ADU vs Exp
#d dark current
#e ADU/s darks
#f Flux vs ADU
#h Flux vs Exp
#l toggle line
#q quit
#r Read noise
#t Flux vs UT
#u Exp vs UT
#v Variance vs ADU
#x Gain vs ADU
#y ADU vs UT
#z Gain vs Exp
#@ toggle average mode
#% toggle percent mode
#. toggle points
        case '?':
          {
          print("                           CURSOR KEY COMMAND SUMMARY")
          print(" ")
          print("? Help             h Flux vs Exp       u Exp vs UT        @ toggle average mode")
          print("a ADU vs Exp       l toggle line       v Variance vs ADU  % toggle percent mode")
          print("d dark current     q quit              x Gain vs ADU      . toggle points")
          print("e ADU/s darks      r Read noise        y ADU vs UT")
          print("f Flux vs ADU      t Flux vs UT        z Gain vs Exp")
          }
        # end of switch
        }

        # reset plot parameters
        pltpar.pointmode = no
        pltpar.crvcolor = INDEF
        dvpar.append = no

        if ( replot )
          {
          replot = no
          key = lastkey
          }
        else
          {
          # read graphics cursor to determine what to plot next ...
          if ( fscan(gcur, gx, gy, gwcs, cmd) != EOF )
            {
            # find the key pressed
            key = substr(cmd, 1, 1)
            }
          else
            {
            # time to leave
            key = "q"
            }
          # end of if replot
          }
      # end of while
      }
    # end of if plot
    }

ende: print("The End")
  # delete temporary files
  delete(table,yes,verify=no)
  delete(tabd1,yes,verify=no)
  delete(tabd2,yes,verify=no)
  delete(tabr1,yes,verify=no)
  delete(tabr2,yes,verify=no)

  delete(ptable,yes,verify=no)
  delete(ptabd1,yes,verify=no)
  delete(ptabd2,yes,verify=no)
  delete(stats,yes,verify=no)

end
